#!/bin/bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG

set -eu

trap __exit EXIT

__iptables() {
	chroot $CHROOTDIR iptables "$@"
}

billing_redirect() {
	# shellcheck disable=SC2154
	[ "${billing['negbal']}" != 1 ] && return 0
	[ -z "${billing['negbal_ip']}" ] && return 0
	[ -z "${billing['negbal_url']}" ] && return 0
	if [ "${billing['justsync']}" != '1' ]; then
		iptables -N bill_http 2>/dev/null || true
		iptables -N bill_https_dst 2>/dev/null || true
		iptables -N bill_dns 2>/dev/null || true
		iptables -N bill_other_traffic 2>/dev/null || true
		insert_negbal_rule
		insert_blocked_rule
		if [ "${billing['nocheckauth']}" = '0' ]; then
			insert_noauth_rule
		fi
		iptables -I http -p tcp --tcp-flags PSH PSH -j bill_http

		if [ -n "${billing['subnet']}" ]; then
			iptables -I bill_https_dst -s "${billing['subnet']}" -d "${billing['subnet']}" -j DROP
		fi
		iptables -I https_dst -j bill_https_dst
		# shellcheck disable=SC2154
		if [ "${billing['other_traffic']:-0}" == '1' ] && [ "${filter['cardsharing']:-}" = '1' ]; then
			if [ -n "${billing['subnet']}" ]; then
				iptables -I bill_other_traffic -s "${billing['subnet']}" -d "${billing['subnet']}" -j DROP
			fi
			iptables -I other_traffic -j bill_other_traffic
		fi

		if $BINDIR/dnsenabled; then
			iptables -I dns -j bill_dns
		fi
	fi
}

ipset_prepare() {
	if [ "${billing['negbal']}" = '1' ]; then
		if [ -n "${billing['subnet']}" ]; then
			ipset create auth bitmap:ip,mac range "${billing['subnet']}" 2>/dev/null || true
		else
			ipset create auth iphash 2>/dev/null || true
		fi
		ipset flush auth
		for ipset in negbal blocked; do
			ipset create $ipset iphash 2>/dev/null || true
			ipset flush $ipset
		done
		if [ -s $TMPDIR/billing_redirect_ipset.save ]; then
			ipset restore < $TMPDIR/billing_redirect_ipset.save
			rm -f $TMPDIR/billing_redirect_ipset.save
		fi
	fi
}


insert_negbal_rule() {
	local url="${billing['negbal_url']}"
	[ "${url// /}" = "" ] && return 0
	echo  "- Добавляется правило редиректа для негбала... "
	__iptables -I bill_http ! -d "${billing['negbal_ip']}" -p tcp -m set --set negbal src -j FORBIDDEN --forbidden-url "$url"
	iptables -A bill_https_dst ! -d "${billing['negbal_ip']}" -p tcp -m set --set negbal src -j REJECT --reject-with tcp-reset
	if [ -n "${billing['subnet']}" ]; then
		iptables -A bill_other_traffic -s "${billing['subnet']}" ! -d "${billing['subnet']}" -p tcp -m set --set negbal src -j REJECT --reject-with tcp-reset
	else
		iptables -A bill_other_traffic ! -d "${billing['negbal_ip']}" -p tcp -m set --set negbal src -j REJECT --reject-with tcp-reset
	fi
	if [ "${filter['dns']:-0}" = '1' ]; then
		iptables -I bill_https_dst -d "${billing['negbal_dns_ip']}" -j DROP
		__iptables -I bill_dns -p udp -m set --set negbal src -j DNSREPLY --dnsreply-ip "${billing['negbal_dns_ip']:-127.0.0.1}"
	fi
}

insert_blocked_rule() {
	local url="${billing['blocked_url']}"
	[ "${url// /}" = "" ] && return 0
	echo  "- Добавляется правило редиректа для заблокированных абонентов... "
	__iptables -I bill_http ! -d "${billing['blocked_ip']}" -p tcp -m set --set blocked src -j FORBIDDEN --forbidden-url "$url"
	iptables -A bill_https_dst ! -d "${billing['blocked_ip']}" -p tcp -m set --set blocked src -j REJECT --reject-with tcp-reset
	if [ -n "${billing['subnet']}" ]; then
		iptables -A bill_other_traffic -s "${billing['subnet']}" ! -d "${billing['subnet']}" -p tcp -m set --set blocked src -j REJECT --reject-with tcp-reset
	else
		iptables -A bill_other_traffic ! -d "${billing['blocked_ip']}" -p tcp -m set --set blocked src -j REJECT --reject-with tcp-reset
	fi
	if [ "${filter['dns']:-0}" = '1' ]; then
		iptables -I bill_https_dst -d "${billing['blocked_dns_ip']}" -j DROP
		__iptables -I bill_dns -p udp -m set --set blocked src -j DNSREPLY --dnsreply-ip "${billing['blocked_dns_ip']:-127.0.0.1}"
	fi
}

insert_noauth_rule() {
	local url="${billing['noauth_url']}"
	[ "${url// /}" = "" ] && return 0
	echo "- Добавляется правило редиректа для неавторизованных абонентов... "
	if [ -n "${billing['subnet']}" ]; then
		__iptables -I bill_http ! -d "${billing['noauth_ip']}" -p tcp -m set ! --set auth src,src -j FORBIDDEN --forbidden-url "$url"
		iptables -A bill_https_dst -s "${billing['subnet']}" ! -d "${billing['subnet']}" -p tcp -m set ! --set auth src,src -j REJECT --reject-with tcp-reset
		iptables -A bill_other_traffic -s "${billing['subnet']}" ! -d "${billing['subnet']}" -p tcp -m set ! --set auth src,src -j REJECT --reject-with tcp-reset
		if [ "${filter['dns']:-0}" = '1' ]; then
			iptables -I bill_https_dst -p tcp -d "${billing['noauth_dns_ip']}" -j DROP
			__iptables -I bill_dns -p udp -m set ! --set auth src,src -j DNSREPLY --dnsreply-ip "${billing['noauth_dns_ip']:-127.0.0.1}"
		fi
	else
		__iptables -I bill_http ! -d "${billing['noauth_ip']}" -p tcp --tcp-flags PSH PSH -m set ! --set auth src -j FORBIDDEN --forbidden-url "$url"
		iptables -A bill_other_traffic ! -d "${billing['noauth_ip']}" -p tcp -m set ! --set auth src -j REJECT --reject-with tcp-reset
		if [ "${filter['dns']:-0}" = '1' ]; then
			__iptables -I bill_dns -p udp -m set ! --set auth src -j DNSREPLY --dnsreply-ip "${billing['noauth_dns_ip']:-127.0.0.1}"
		fi
	fi
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	if [ -x "$hook" ]; then
		# shellcheck disable=SC1090
		. "$hook" __include
	fi
}

start() {
	ipset_prepare
	billing_redirect
}

stop() {
	if ipset save | egrep ^add.*'(auth|negbal|blocked)' > $TMPDIR/billing_redirect_ipset.save.tmp; then
		mv $TMPDIR/billing_redirect_ipset.save{.tmp,}
	fi
	return 0
}

use_hook
case "$1" in
	start )
		start
		;;
	stop )
		stop
		;;
	* )
		log "$@" no supported
		;;
esac
