#!/bin/bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG

set -eu

HTTP_PROCFILE=/proc/net/ipt_reductor/block_list
HTTP_LOCKFILE=$LOCKDIR/ipt_reductor.lock
HTTP_FD=10

DNS_PROCFILE=/proc/net/ipt_dnsmatch/block_list
DNS_LOCKFILE=$LOCKDIR/ipt_dnsmatch.lock
DNS_FD=11

SNI_PROCFILE=/proc/net/ipt_snimatch/block_list
SNI_LOCKFILE=$LOCKDIR/ipt_snimatch.lock
SNI_FD=12

LOCKFILE_URL=$LOCKDIR/url.lock
LOCKFILE_DOMAIN=$LOCKDIR/domain.lock

FD_URL=3
FD_DOMAIN=4

export CUSTOM_EXIT=1

__custom_exit() {
	release_lock $HTTP_LOCKFILE $HTTP_FD
	release_lock $DNS_LOCKFILE $DNS_FD
	release_lock $SNI_LOCKFILE $SNI_FD
	release_lock $LOCKFILE_URL $FD_URL || true
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN || true
}

trap __exit EXIT

prepare() {
	mkdir -p $TMPDIR $LOCKDIR
}

# теперь можем спокойно использовать этот враппер везде, скорость load НЕ ВАЖНА из-за double buffer
write2proc() {
	echo "$@" > $PROCLIST
}

load_url_and_signatures_to_proc() {
	local failed=0
	local list_id="$1"
	local input_file="$2"
	echo >> $LOGFILE || true
	log "INFO: начинаем загрузку URL в ядро" >> $LOGFILE || true
	while read sign url; do
		if ! write2proc "$sign $list_id $url"; then
			log "ERROR: не удалось добавить _ $sign $list_id $url _"
			((failed++)) || true
		fi
	done < "$input_file"
	log "Загрузка URL в ядро закончена" >> $LOGFILE || true
	[ "$failed" -gt '0' ] && log "ERROR: $failed URL не было добавлено" | tee -a $LOGFILE || true
	echo >> $LOGFILE || true
	return 0
}

load_signatures_and_cache() {
	write2proc clear
	local signatures=$CACHEDIR/signatures.cache
	local loadfile=$TMPDIR/signatures_and_urls.tmp
	set_lock $HTTP_LOCKFILE 60 $HTTP_FD 0 || return 0
	set_lock $LOCKFILE_URL 60 $FD_URL 0 || return 0
	log "Заливаем URL в HTTP-матчер"
	for list in $(url_lists); do
		log "  $list"
		# -t <TAB>, not a few spaces
		if ! LC_CTYPE='' LANG='' join -t '	' -1 1 -2 2 -o "2.1 1.1" $list $signatures > $loadfile; then
			log "WARNING: Ошибка при обработке списка $list, пробуем подождать пару секунд и повторить..." | tee -a $LOGFILE || true
			log "ERROR: LC_CTYPE='' LANG='' join -t '      ' -1 1 -2 2 -o '2.1 1.1' $list $signatures > $loadfile"
			log "WARNING: Пытаемся обработать списки, мы обязаны любым способом запуститься с загруженными списками."
			$BINDIR/prepare_lists.sh
			sync
			log "Повторная попытка загрузки:"
			log "  $list"
			if ! LC_CTYPE='' LANG='' join -t '	' -1 1 -2 2 -o "2.1 1.1" $list $signatures > $loadfile; then
				log "ERROR: Ошибка при обработке списка $list" | tee -a $LOGFILE || true
				continue
			fi
		fi
		grep -v ^clear $TMPDIR/signatures_and_urls.tmp > $TMPDIR/signatures_and_urls || true
		list_id=${list##*/}
		[ "$list_id" = "url_http.load" ] && list_id=0 || true

		load_url_and_signatures_to_proc $list_id $TMPDIR/signatures_and_urls
	done
	release_lock $LOCKFILE_URL $FD_URL
	write2proc switch
	release_lock $HTTP_LOCKFILE $HTTP_FD
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	# shellcheck disable=SC1090
	[ ! -x "$hook" ] || . "$hook"
}

client_prepare_hook() {
	return 0
}

prepare_lists_if_need() {
	if [ ! -s $LISTDIR/rkn/rkn.url_http ] && [ ! -s $LISTDIR/rkn/rkn.ip_https ] && [ -s $DUMPXML ]; then
		log "Разбираем единый реестр запрещённых сайтов, так-как один из необходимых списков пустой."
		$BINDIR/support_info.sh lists_size | grep rkn | while IFS= read line; do log "$line"; done
		$BINDIR/dump_parser.sh
	fi

	if [ ! -s $LISTDIR/load/url_http.load ]; then
		log "Агрегируем списки, так как url_http.load пустой"
		$BINDIR/support_info.sh lists_size | grep load | while IFS= read line; do log "$line"; done
		$BINDIR/prepare_lists.sh
	fi
}

load_sni_list() {
	local prefix="$1"
	local id="$2"
	local list="$LISTDIR/load/$prefix.load"
	log "  $list"
	if ! [ -s "$list" ]; then
		log "WARNING: Пустой $list"
		return 0
	fi
	python -u $BINDIR/sniload.py $list $id | while read line; do
		echo "$line" > $SNI_PROCFILE
	done
}

load_sni() {
	local prefix
	$BINDIR/dnsenabled --sni || return 0
	set_lock $SNI_LOCKFILE 60 $SNI_FD 0 || return 0
	set_lock $LOCKFILE_DOMAIN 60 $FD_DOMAIN 0 || return 0
	log "Загрузка доменов в snimatch"
	echo "clear" > $SNI_PROCFILE
	# для SNI не имеет значения id списка, так что всегда 0
	load_sni_list domain_mask 0
	if $BINDIR/dnsenabled --separate-exact-and-mask; then
		load_sni_list domain_exact 0
	fi
	if $BINDIR/dnsenabled --proxy-https; then
		load_sni_list domain_proxy 0
	fi
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN
	echo "switch" > $SNI_PROCFILE
	release_lock $SNI_LOCKFILE $SNI_FD
}

load_dns_list() {
	local list="$1"
	local list_id="${2:-0}"
	local exact="${3:-0}"
	log "  $list"
	if [[ "$list" != $LISTDIR/load/domain_*.load ]]; then
		list_id="${list##*/}"
		read _ _ _ exact _ <<< "$(grep "^$list_id " $HOOKDIR/custom_bindings_dns)" || return 0
	fi
	if ! [ -s $list ]; then
		log "WARNING: Пустой $list"
		return 0
	fi
	python -u $BINDIR/dnsload.py "$list" "$list_id" "$exact" | while read domain; do
		echo -n "$domain" > $DNS_PROCFILE
	done

}

load_dns() {
	local prefix
	$BINDIR/dnsenabled || return 0
	set_lock $DNS_LOCKFILE 60 $DNS_FD 0
	set_lock $LOCKFILE_DOMAIN 60 $FD_DOMAIN 0 || return 0
	log "Загрузка доменов в dnsmatch"
	echo "clear" > $DNS_PROCFILE
	load_dns_list $LISTDIR/load/domain_mask.load 0 0
	if $BINDIR/dnsenabled --separate-exact-and-mask; then
		load_dns_list $LISTDIR/load/domain_exact.load 1000 1
	fi
	if $BINDIR/dnsenabled --proxy-https; then
		load_dns_list $LISTDIR/load/domain_proxy.load 2000 0
	fi
	if $BINDIR/dnsenabled --proxy-hsts; then
		load_dns_list $LISTDIR/load/domain_hsts.load 3000 0
	fi
	#TODO: тестировать
	for list in $(domain_custom_lists); do
		load_dns_list $list
	done
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN
	echo "switch" > $DNS_PROCFILE
	release_lock $DNS_LOCKFILE $DNS_FD
}

main() {
	prepare
	prepare_lists_if_need
	client_prepare_hook
	$BINDIR/load_ip.sh
	load_signatures_and_cache
	load_dns
	load_sni
}

use_hook
main "$@"
