#!/bin/bash

# Здесь не стоит делать set -eu
# Здесь не стоит менять названия цепочек

. /usr/local/Reductor/etc/const
. $CONFIG

declare -a rules
rules=(
	"-p tcp -j REJECT --reject-with tcp-reset"
	"-p udp -j REJECT --reject-with icmp-port-unreachable"
	"-j REJECT --reject-with icmp-proto-unreachable"
)

chain=reductor_maintain

lock() {
	if $BINDIR/dnsenabled --revisor; then
		log "Полная блокировка доступа в интернет для ${filter['revisor_ip']} на время обслуживания."
	else
		log "Полная блокировка доступа в интернет на время обслуживания."
	fi
	iptables -N $chain &>/dev/null || true
	iptables -F $chain # because ip can be added or changed
	for rule in "${rules[@]}"; do
		user=""
		if $BINDIR/dnsenabled --revisor; then
			user="-s ${filter['revisor_ip']}"
		fi
		iptables -A $chain $user $rule
	done
	iptables -I FORWARD $user -j $chain
}

unlock() {
	is_locked || return 0
	if $BINDIR/dnsenabled --revisor; then
		log "Снятие полной блокировки для ${filter['revisor_ip']}, обслуживание сервера завершено."
	else
		log "Снятие полной блокировки, обслуживание сервера завершено."
	fi
	iptables -D FORWARD -j $chain &>/dev/null || true
	for _ in {1..3}; do # Заботимся о пользователях, баги не должны оставить их без инета
		for rule in "${rules[@]}"; do
			user=""
			iptables -D $chain $user $rule &>/dev/null || true
			if $BINDIR/dnsenabled --revisor; then
				user="-s ${filter['revisor_ip']}"
				iptables -D $chain $user $rule &>/dev/null || true
			fi
		done
	done
}

is_locked() {
	iptables -C FORWARD -j $chain &>/dev/null && log "В текущий момент проводится техническое обслуживание сервера, интернет заблокирован всем." && return 0
	if ! $BINDIR/dnsenabled --revisor; then
		log "В текущий момент технического обслуживания не производится."
		return 1
	fi
	if iptables -C FORWARD -s ${filter['revisor_ip']} -j $chain &>/dev/null; then
		log "В текущий момент производится техническое обслуживание, интернет заблокирован только для  ${filter['revisor_ip']}"
		return 0
	fi
}

unlock_delayed() {
	$BINDIR/dnsenabled --revisor && return 0
	local warning="Система находится на обслуживании. Если это продлится ещё 30 секунд, блокировка будет снята."
	local alarm="Снимаю полную блокировку, несмотря на то, что обслуживание ещё не закончилось. Скорее всего произошло зависание. Нужно разобраться с Carbon Reductor."
       	if is_locked; then
		log "$warning"
		sleep 30
		if is_locked; then
			log "$alarm"
			log "Администраторам будет послано уведомление об этом."
			echo "$alarm" > $TMPDIR/alarm.unlock
			send_alarm $TMPDIR/alarm.unlock
			unlock
		fi
	fi
}

main() {
	case "$1" in
		lock | unlock | unlock_delayed | is_locked )
			"$1"
			;;
		* )
			log "Вызвана неизвестная команда: $*"
			exit 1
	esac
}

main "$@"
