#!/bin/bash

if [[ "$(readlink /proc/$$/fd/1)" = '/dev/pts/'* ]]; then
	. /etc/init.d/functions
else
	SETCOLOR_NORMAL=''
	SETCOLOR_FAILURE=''
	SETCOLOR_SUCCESS=''
fi

. /usr/local/Reductor/etc/const
. $CONFIG
. $BINDIR/mirror_info.sh include

error_count=0
error_only=0
[ "${1:-}" = 'erroronly' ] && error_only=1 || true

mirror() {
	local rc=0
	mirror_devices="$(mirror_devices)"
	if [ ! -n "${mirror_devices:-}" ]; then
		echo "Не настроено зеркало трафика"
		return 1
	fi
	for dev in $mirror_devices; do
		count=100
		file=/tmp/mirror.pcap.$dev
		if ! timeout -s 15 2s tcpdump -q -c 1 -i $dev -w $file 2>/dev/null; then
			((rc++)) || true
			echo "Не удалсь проанализировать $dev, вероятно там нет трафика"
			continue
		fi
		rm -f $file
		gather $file
		if ! proto_analyze_human tcp 80 > $TMPDIR/tcp80; then
			echo "Проверяем наличие http"
			echo "$(<$TMPDIR/tcp80)"
			echo
			((rc++)) || true
		fi
		if ! proto_analyze_human tcp 443 > $TMPDIR/tcp443; then
			((rc++)) || true
			echo "Проверяем наличие https в обе стороны"
			echo "$(<$TMPDIR/tcp443)"
			echo
		fi
		if ! proto_analyze_human udp 53 > $TMPDIR/udp53; then
			((rc++)) || true
			echo "Проверяем наличие DNS"
			echo "$(<$TMPDIR/udp53)"
			echo
		fi
		# echo "Проверяем наличие IPv6"
		# echo "Проверяем наличие vlan"
	done
	return $rc
}

rkn() {
	if [ "${autoupdate['enabled']:-0}" = '0' ] && [ "${autoupdate['own']:-0}" != '1' -a ! -s $PRIVATE_KEY ]; then
		[ "${autoupdate['enabled']}" = '1' ] || echo "Автообновление отключено"
		[ "${autoupdate['own']:-0}" = '1' ] || echo "Выгрузка с сервера РКН"
		[ -s $PRIVATE_KEY ] || echo "Нет закрытого ключа"
		return 1
	fi
}

https() {
	[ "${filter['https_by_ip']:-0}" == '1' ]
}

dns() {
	[ "${filter['dns']:-0}" == '1' ]
}

ip_block() {
	if ! [ -x $HOOKDIR/events.sh ]; then
		echo "Необходимо настроить интеграцию с маршрутизатором для блокировки некоторых ресурсов по IP целиком"
		return 1
	fi
}

security() {
	local rc=0
	if ! grep -q ^Port /etc/ssh/sshd_config; then
		echo "Стандартный порт SSH"
		echo
		((rc++)) || true
	fi

	if [ "${admin['ip']:-}" = '' ]; then
		echo "Необходимо указать в настройках IP адреса администраторов, которым разрешён доступ к веб-интерфейсу."
		echo
		((rc++)) || true
	fi

	if [ "${reductorupdate['branch']:-devel}" != 'master' ]; then
		echo "Используется не master-ветка продукта (${reductorupdate['branch']}). Укажите reductorupdate['branch']='master' и обновитесь."
		echo
		((rc++)) || true
	fi
	return $rc
}

carbonsoft() {
	if ! [ "${misc['admin_notify']:-0}" == '1' ]; then
		echo "Отключены уведомления администратора о критических проблемах с помощью редиректа"
		return 1
	fi
	return 0
}

nic() {
	:
}

check() {
	local rc
	rc=0
	"$@" > $TMPDIR/need_fix.sh || rc=$? || true
	if [ "$rc" = 0 ]; then
		[ "$error_only" = '1' ] && return 0 || true
		$SETCOLOR_SUCCESS
	else
		$SETCOLOR_FAILURE
	fi
	echo "# $1"
	$SETCOLOR_NORMAL
	if [ "$rc" == 0 ]; then
		echo
		return $rc
	fi
	echo 
	sed 's/^/    /' $TMPDIR/need_fix.sh 
	echo 
	((error_count++)) || true
}

performance() {
	local rc=0
	if [ "${filter['dns_log_level']}" -gt 0 ]; then
		((rc++)) || true
		echo "Нужно снизить уровень логирования DNS до 0"
		echo
	fi

	if [ "${filter['forbidden_log']:-1}" != '0' ]; then
		((rc++)) || true
		echo "Рекомендуется отключить логирование HTTP модуля"
		echo
	fi

	if [ "${filter['notrack']:-0}" != 1 ]; then
		((rc++)) || true
		echo "Необходимо настроить NOTRACK. Для этого необходимо, чтобы в зеркале трафика отсутствовал трафик самого Carbon Reductor."
		echo
	fi
	return $rc
}

main() {
	error_count=0
	check mirror
	check rkn
	check nic
	check https
	check dns
	check ip_block
	check security
	check carbonsoft
	check performance
	return $error_count
}

main "${@:-}"
