#!/usr/bin/env bash

. /usr/local/Reductor/etc/const

set -eu

domain_list_merge() {
	local prefix_output="$1"
	local prefix_remove="$2"
	local tmpfile=$LISTDIR/tmp/domain_${prefix_remove}_and_${prefix_output}.tmp
	sort -u $LISTDIR/load/domain_{$prefix_remove,$prefix_output}.load > $tmpfile
	mv -f $tmpfile $LISTDIR/load/domain_${prefix_output}.load
	rm -f $LISTDIR/load/domain_${prefix_remove}.load
}

dns_separate_exact_and_mask() {
	if ! $BINDIR/dnsenabled --separate-exact-and-mask; then
		log "Отключение точной фильтрации доменов (всё фильтруется по маскам)"
		domain_list_merge mask exact
	fi
}

dns_proxy_url_https() {
	if ! $BINDIR/dnsenabled --proxy-https; then
		log "Отключение отправки HTTPS-ресурсов заблокированных по URL в прокси-сервер по DNS"
		domain_list_merge mask proxy
	fi
}

dns_proxy_hsts() {
	local pseudo_url_list
	local temporary_url_list

	if ! $BINDIR/dnsenabled --proxy-hsts; then
		log "Отключение отправки ресурсов, использующих HSTS, в прокси-сервер по DNS"
		domain_list_merge mask hsts
		return
	fi

	log "Генерируем списки для фильтрации HSTS-ресурсов"
	for proto in http https; do
		pseudo_url_list="$LISTDIR/tmp/url_${proto}.domain_hsts"
		temporary_url_list="$LISTDIR/tmp/url_${proto}.domain_hsts.result"
		sed -E "s|^.*$|$proto://&/|" $LISTDIR/load/domain_hsts.load > $pseudo_url_list
		fgrep -f $pseudo_url_list $LISTDIR/load/url_${proto}.load > $temporary_url_list || true
		[ "$proto" != 'https' ] || continue
		sed -e "s/$proto:/https:/" < $temporary_url_list > $temporary_url_list.ok
		mv -f $temporary_url_list{.ok,}
	done
	cat $LISTDIR/tmp/url_http{,s}.domain_hsts.result > $LISTDIR/tmp/url_hsts.load
	$BINDIR/sort.sh $LISTDIR/tmp/url_hsts.load
	mv -f $LISTDIR/{tmp,load}/url_hsts.load
}

main() {
	log "Обработка списков согласно опциям в конфиге"
	dns_separate_exact_and_mask
	dns_proxy_url_https
	dns_proxy_hsts
}

main "$@"
