#!/usr/bin/env bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG

LOCKFILE_DOMAIN=$LOCKDIR/domain.lock
LOCKFILE_IP=$LOCKDIR/ip.lock
LOCKFILE_RESOLVER=$LOCKDIR/resolver.lock

FD_DOMAIN=3
FD_IP=4
FD_RESOLVER=5

set -eu

trap __exit EXIT

use_hook() {
	local hook="$HOOKDIR/${0##*/}"
	# shellcheck disable=SC1090
	[ ! -x "$hook" ] || . "$hook"
}

all_domains() {
	local all_domains_file="$1"
	set_lock $LOCKFILE_DOMAIN 60 $FD_DOMAIN
	find $LISTDIR/load/ -maxdepth 1 -type f -name "domain_*.load" | egrep -v "domain_(hsts|proxy).load" | xargs cat | sort -u > $all_domains_file.tmp
	mv -f $all_domains_file.tmp $all_domains_file
	ln -sf $all_domains_file $LISTDIR/https.resolv
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN
}

resolv() {
	local all_domains_file="$1"
	local ipv6_support="$2"
	local DNS_BLACKHOLE_IP="127.0.0.1"
	if $BINDIR/dnsenabled; then
		DNS_BLACKHOLE_IP="$($BINDIR/dnsenabled --nginx-ip)"
	fi
	export DNS_BLACKHOLE_IP=$DNS_BLACKHOLE_IP
	log "updating domains"
	set +eu
	# shellcheck disable=SC1090
	. $VENVBIN/activate
	set -eu
	$VENVBIN/cache_cleanup.py < $all_domains_file
	$VENVBIN/ip_update.py $ipv6_support < $all_domains_file >&2
	$VENVBIN/dns_db.py --convert $CACHEDIR/https_resolv.{json,pickle}
	set +eu
	deactivate
	set -eu
}

query() {
	local ipv6_support="$1"
	set_lock $LOCKFILE_DOMAIN 60 $FD_DOMAIN
	set_lock $LOCKFILE_IP 60 $FD_IP
	for prefix in $($BINDIR/prepare_lists.sh --show-list domains); do
		for proto in ip ip6; do
			[ "$ipv6_support" == '1' ] || [ "$proto" != 'ip6' ] || continue
			[ -s "$LISTDIR/load/$prefix.load" ] || continue
			$VENVBIN/query_cache.py $proto < $LISTDIR/load/$prefix.load > $LISTDIR/resolver/$prefix.${proto}_https_plus
		done
	done
	release_lock $LOCKFILE_IP $FD_IP
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN
}

main() {
	# shellcheck disable=SC2154
	local ipv6_support="${filter['ipv6']:-0}"
	local all_domains_file="$LISTDIR/tmp/domains.all"
	all_domains "$all_domains_file"
	[ "${filter['https_by_nslookup']}" != '0' ] || return 0
	set_lock $LOCKFILE_RESOLVER 60 $FD_RESOLVER
	resolv "$all_domains_file" "$ipv6_support"
	query $ipv6_support
	release_lock $LOCKFILE_RESOLVER $FD_RESOLVER
	$BINDIR/prepare_lists.sh --only-ip
	$BINDIR/load_ip.sh
	find /tmp/ -type f -name "https_resolv.json.*" -delete
}

use_hook
main "$@"
