#!/usr/bin/python
# coding=utf8

# -*- author: oleg -*-

import base64
import time
import zipfile
import os
import sys
import socket
import urllib2
import random
import suds

SERVER_SIDE_EXCEPTIONS = (urllib2.URLError, socket.error, suds.transport.TransportError)
MAX_TIMEOUT = 150


def make_client():
    rkn_url = "http://vigruzki.rkn.gov.ru/services/OperatorRequest/?wsdl"
    client = None
    timeout = 10
    u_print(u"Устанавливаем соединение с сервером")
    while not client:
        try:
            if os.environ.get('SUDS_NO_CACHE'):
                client = suds.client.Client(rkn_url, cache=None)
            else:
                client = suds.client.Client(rkn_url)
            u_print(u"Соединение установлено")
            return client
        except SERVER_SIDE_EXCEPTIONS as err:
            print err
            timeout += random.randint(5, 15)
            if timeout > MAX_TIMEOUT:
                break
            u_print(u"Повторная попытка через {0} секунд.".format(timeout))
            time.sleep(timeout)
    raise err

CHROOTDIR = "/usr/local/Reductor/reductor_container/gost-ssl/php/"
ZIP_FILE = CHROOTDIR + "register.zip"
REQUEST_FILE = "request.xml"
OUTPUT = "dump.xml"
MAIN_DOWNLOAD = True
if len(sys.argv) == 2 and sys.argv[1] == '--second':
    MAIN_DOWNLOAD = False
if not MAIN_DOWNLOAD:
    REQUEST_FILE = "request_second.xml"
    print "Performing second request"
SIGNATURE = REQUEST_FILE + ".sign"


def u_print(string):
    print string.encode('utf8')


def prepare():
    if not MAIN_DOWNLOAD:
        return
    u_print(u'Удаляем предыдущий архив.')
    if os.path.exists(ZIP_FILE):
        os.unlink(ZIP_FILE)


def get_code(client):
    u_print(u'Запрашиваем идентификатор запроса:')
    request = sendRequest(client)
    if 'code' in request:
        u_print(u'Идентификатор:' + str(request['code']))
        return request['code']
    u_print(u'Не удалось получить идентификатор запроса')
    if 'resultComment' in request:
        print request['resultComment']
    print request
    exit(1)


def file_data_base64(filename):
    filename = open(filename, "rb")
    data = filename.read()
    filename.close()
    return base64.b64encode(data)


def result2dict(result):
    """ Нельзя здесь возвращать None """
    return dict(((k, v.encode('utf-8')) if isinstance(v, suds.sax.text.Text) else (k, v)) for (k, v) in result)


def sendRequest(client):
    u_print(u'Отправка запроса')
    xml = file_data_base64(CHROOTDIR + REQUEST_FILE)
    sign = file_data_base64(CHROOTDIR + REQUEST_FILE + ".sign")
    result = client.service.sendRequest(xml, sign, "2.2")
    if not result:
        u_print(u'Отсутствует ответ от Роскомнадзора')
        exit(2)
    return result2dict(result)


def getResult(client, code):
    u_print(u' Запрос архива (' + str(code) + ')')
    result = client.service.getResult(code)
    return result2dict(result)


def download(client, code, sleep_time=30):
    u_print(u'Скачивание архива:')
    result = getResult(client, code)
    line = u''
    while result['resultCode'] <= 0:
        if sleep_time > MAX_TIMEOUT:
            u_print(u'Превышено количество запросов к серверу Роскомнадзора')
            exit(3)
        u_print(u'  пауза (' + str(sleep_time) + u') секунд..')
        time.sleep(sleep_time)
        sleep_time += random.randint(5, 15)
        try:
            result = getResult(client, code)
        except SERVER_SIDE_EXCEPTIONS as err:
            print err
            continue
        if 'resultComment' in result:
            line = u'  Комментарий: ' + result['resultComment'].decode('utf-8')
        if 'inn' in result:
            line = line + u'. Инн: ' + result['inn'].decode('utf-8')
        if 'resultCode' in result:
            u_print(line + ' (' + str(result['resultCode']) + ')')
            if result['resultCode'] < 0:
                u_print(u' Запрос списков завершён с ошибкой')
                exit(4)
        if 'registerZipArchive' in result:
            u_print(u' zip-архив присутствует в ответе')
            continue
    if result and result['resultCode'] == 1:
        u_print(u' Запрос списков завершён успешно')
    return result


def unpack(result):
    u_print(u'Распаковка архива')
    if not 'registerZipArchive' in result:
        u_print(u'В ответе РосКомНадзора отсутствует архив.')
        exit(5)
    os.chdir(CHROOTDIR)
    if not MAIN_DOWNLOAD:
        os.chdir("/tmp/")
    with open(ZIP_FILE, 'w') as f:
        f.write(base64.b64decode(result['registerZipArchive']))
    if zipfile.ZipFile(ZIP_FILE).extract('dump.xml'):
        u_print(u'Список запрещённых сайтов успешно извлечён')


def write_code(code):
    with open(CHROOTDIR + 'dump.xml.code', 'w') as f:
        f.write(code + '\n')


def main():
    prepare()
    client = make_client()
    code = get_code(client)
    result = download(client, code)
    unpack(result)
    write_code(code)


if __name__ == '__main__':
    main()
