#!/bin/bash

. /usr/local/Reductor/etc/const
. "$CONFIG"

LOCKDIR=/var/lock/reductor/
LOCKFILE=$LOCKDIR/rkn_download.lock

CUSTOM_EXIT=1
__custom_exit() {
	flock -u 3
}

trap __exit EXIT

prepare() {
	mkdir -p $LOCKDIR
	exec 3>$LOCKFILE
	log "Захватываем lock выгрузки списков РКН..."
	if ! flock -w 60 -x 3; then
		log "ERROR: Не смогли захватить lock."
		pstree -upal | while IFS= read -r line; do
			log "$line"
		done
		exit 1
	fi
	log "Выгружаем список РКН."
	if [ "${autoupdate['enabled']}" = '0' ]; then
		log "Обновление списков пропущено ибо отключено в меню"
		if [ "${autoupdate['own']}" = '0' ]; then
			dump="${1:-$DUMPXML}"
			if [ -f $dump ]; then
				log "Но так как dump.xml лежит на месте - разберём его"
				$BINDIR/dump_parser.sh
			fi
			exit 0
		fi
	fi
	if [ "$(pgrep rkn_download.sh)" != "$$" ]; then
		log "Обновление списков уже запущено, несмотря на flock. Невозможная ситуация, которой мы пытаемся избежать."
		for pid in $(pgrep rkn_download.sh) $(pgrep reductor_chroot); do
			if [ "$pid" != "$$" ]; then
				log "Завершаем процесс $pid $(sed -e 's/.*\///g' /proc/$pid/cmdline 2>/dev/null)"
				kill -KILL $pid
			fi
		done
		log "Процесс $$ продолжает выполнение"
	fi
	echo "Пожалуйста, не прерывайте работу скрипта (ctrl+c, или закрытие окна), он выполняется достаточно долго (1-5 минут)"
}

check_private_key() {
	if [ "${autoupdate['skip_sign_request']:-0}" = '1' -o "${autoupdate['own']:-0}" = '1' ]; then
		return 0
	fi
	if [ ! -s "$PRIVATE_KEY" ]; then
		log "Необходимо добавить экспортированный сертификат в каталог $USERDIR/" >&2
		exit 1
	fi
}

check_activate() {
	rm -f $TMPDIR/reductor_report.*
	if [ "$($BINDIR/modules_ctl get_state_human ipt_reductor)" != 'Активирован' ]; then
		$BINDIR/reg_util activate
		$BINDIR/registration.sh activation
	fi

	if [ "$($BINDIR/modules_ctl get_state_human ipt_reductor)" != 'Активирован' ]; then
		report=$TMPDIR/reductor_report.$$
		log "Не удалось активироваться!" >> $LOGFILE
		log "Не удалось активироваться!" >&2
		log "Не удалось активироваться при старте выгрузки. " >> $report
		log "Не могу запустить обновление списка на неактивированной копии Carbon Reductor" >&2
		cat $PROCLIST > $report
		send_alarm "$report"
		exit 2
	fi
}

prepare_request() {
	if ! chroot $CHROOTDIR /bin/gen_request.sh; then
		log "ERROR: Can't gen request.xml"
		exit 1
	fi

	if ! chroot $CHROOTDIR /bin/sign_request.sh; then
		log "ERROR: Can't sign request.xml"
		exit 2
	fi

	local second_key=${PRIVATE_KEY//provider/second}
	local chroot_second_key=/gost-ssl/ssl/private/second.key
	local second_request=/gost-ssl/php/request_second.xml

	if ! [ -f $second_config -a $second_key ]; then
		return 0
	fi

	if ! chroot $CHROOTDIR /bin/gen_request.sh /config_second $second_request; then
		 log "ERROR: Can't gen request_second.xml"
	fi

	if ! chroot $CHROOTDIR /bin/sign_request.sh $chroot_second_key $second_request $second_request.sign; then
		 log "ERROR: Can't sign request_second.xml"
	fi
}

chroot_work() {
	local second_key=${PRIVATE_KEY//provider/second}
	local chroot_second_key=$SSLSSLDIR/private/second.key
	local second_config=${CONFIG}_second
	local report=$TMPDIR/rkn_download_report.$$

	mkdir -p $TMPDIR/
	cp -f $CONFIG $CHROOTDIR/
	cp -f ${PRIVATE_KEY} ${SSLSSLDIR}/private/provider.pem
	if [ -f $second_config -a $second_key ]; then
		cp -f $second_config $CHROOTDIR/
		cp -f $second_key $chroot_second_key
	fi

	chroot_mount
	if [ "${autoupdate['skip_sign_request']:-0}" != '1' ]; then
		prepare_request
	fi

	. $CHROOTDIR/virtualenv/bin/activate
	export SUDS_NO_CACHE=0
	log "## Первая попытка (suds cache включен):"
	if ! python -u $BINDIR/rkn_download.py &> $TMPDIR/rkn_download.1; then
		cat $TMPDIR/rkn_download.1
		log "WARNING: Выгрузка не удалась, попробуем через 100 секунд..."
		sleep 100
		export SUDS_NO_CACHE=1
		log "## Вторая попытка (suds cache выключен):"
		if ! python -u $BINDIR/rkn_download.py &> $TMPDIR/rkn_download.2; then
			cat $TMPDIR/rkn_download.2
			log "ERROR: RKN lists update failed"
			{
				log "# ERROR: RKN lists update failed"
				log "## Первая попытка (suds cache включен):"
				cat $TMPDIR/rkn_download.1
				log "## Вторая попытка (suds cache выключен):"
				cat $TMPDIR/rkn_download.2
			} > $report
			send_alarm "$report"
			rm -f "$report"
			exit 3
		else
			cat $TMPDIR/rkn_download.2
		fi
	else
		cat $TMPDIR/rkn_download.1
	fi

	if [ -f $second_config -a $second_key ]; then
		if ! python -u $BINDIR/rkn_download.py --second; then
			log "ERROR: RKN lists update for second provider failed"
			log "ERROR: RKN lists update for second provider failed" > "$report"
			send_alarm "$report"
			rm -f "$report"
		fi
	fi
	deactivate

	if [ ! -f $SSLDIR/php/register.zip ]; then
		log "ERROR: No register.zip found"
		log "ERROR: No register.zip found" > $report
		send_alarm "$report"
		rm -f "$report"
		exit 4
	fi

	if [ ! -f $DUMPXML ]; then
		log "ERROR: No dump.xml found"
		log "ERROR: No dump.xml found" > $report
		send_alarm "$report"
		rm -f "$report"
		exit 5
	fi

	umount -f ${CHROOTDIR}/dev
}


download_from_own_server() {
	echo "- Скачиваем список с ${autoupdate['own_url']}"
	if wget -q -O /tmp/dump.xml "${autoupdate['own_url']}"; then
		if [ -s /tmp/dump.xml ]; then
			mv /tmp/dump.xml $DUMPXML
		else
			echo "Игнорируем пустой файл"
		fi
	else
		echo "Список недоступен, продолжаем использовать старый."
	fi
}

client_pre_hook() {
	return 0
}

client_post_hook() {
	return 0
}

use_hook() {
	hook=$HOOKDIR/${0##*/}
	[ -x $hook ] && . $hook
}

# извлекаем из dump.xml всё что нужно

main() {
	log "Запущено обновление списков РосКомНадзора"
	client_pre_hook
	prepare "$@"
	check_private_key
	check_activate
	if [ "${autoupdate['own']}" = '1' -a -n "${autoupdate['own_url']}" ]; then
		download_from_own_server
	else
		chroot_work
	fi
	$BINDIR/dump_parser.sh
	client_post_hook
	log "Завершено обновление списков РосКомНадзора" >> $LOGFILE
}

use_hook
main "$@"
