#!/bin/bash

. /usr/local/Reductor/etc/const
. /usr/local/Reductor/bin/setup_master_lib.sh
# shellcheck disable=SC1090
. $CONFIG

registration() {
	insert_module xt_reductor
	show_msg "Регистрация. Регномер: $($BINDIR/reg_util create)"
	company_name="$(ask_company_name)"
	email="$(ask_email_adress "Введите e-mail адрес человека для решения ТЕХНИЧЕСКИХ вопросов (системного администратора) компании" "email")"
	phone="$(ask_phone_number "Введите номер телефона человека для решения ТЕХНИЧЕСКИХ вопросов (системного администратора) компании" "phone")"
	email_bookkeeper="$(ask_email_adress "Введите e-mail адрес человека для решения ФИНАНСОВЫХ вопросов (бухгалтера) компании" "email_bookkeeper")"
	phone_bookkeeper="$(ask_phone_number "Введите номер телефона человека для решения ФИНАНСОВЫХ вопросов (бухгалтера) компании" "phone_bookkeeper")"
	email_director="$(ask_email_adress "Введите e-mail адрес человека для решения ОРГАНИЗАЦИОННЫХ вопросов (директора) компании" "email_director")"
	phone_director="$(ask_phone_number "Введите номер телефона человека для решения ОРГАНИЗАЦИОННЫХ вопросов (директора) компании" "phone_director")"
	export company_name email email_director email_bookkeeper phone phone_director phone_bookkeeper
	$BINDIR/reg_util update
	$BINDIR/reg_util activate
	$BINDIR/registration.sh activation
}

rkn_update() {
	local email_regexp="^[a-z0-9_\+-]+(\.[a-z0-9_\+-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*\.([a-z]{2,4})$"
	# shellcheck disable=SC2154
	autoupdate['enabled']="$(question_choice "Включить автоматическое обновление единого реестра запрещённых сайтов?" "1")"
	if [ -z "${autoupdate['operator']}" ]; then
		autoupdate['operator']="$(ask_company_name)"
	fi
	# shellcheck disable=SC2154
	autoupdate['inn']="$(ask_inn)"
	# shellcheck disable=SC2154
	autoupdate['ogrn']="$(ask_ogrn)"
	[ -n "${autoupdate['email']}" ] && autoupdate['email']="$(get_prev_value "email")"
	autoupdate['email']="$(question_inputbox "Регистрация" "Введите e-mail адрес для РКН" "$email_regexp" "${autoupdate['email']}")"
}

rkn_settings() {
	local url_regexp='(http|https)://[^|/"]+'
	case "$(select_rkn_update_method)" in
		pem )
			rkn_update
			;;
		xml )
			autoupdate['skip_sign_request']='1'
			show_msg "Следить за работоспособностью запроса и подписи вы обязуетесь самостоятельно"
			;;
		web )
			autoupdate['own']='1'
			autoupdate['own_url']="$(question_inputbox "Настройки выгрузок единого реестра" "Введите URL c dump.xml на своем сервере" "$url_regexp" "${autoupdate['own_url']}")"
			;;
	esac
	save_and_exit
}

http() {
	show_msg "Настройка фильтрации HTTP"
	# shellcheck disable=SC2154
	filter['forbidden']="$(question_choice "Использовать модуль HTTP-редиректа на страницу-заглушку?\n\nПользователям будет отправляться HTTP 302 + RST." "${filter['forbidden']}")"
	[ "${filter['forbidden']}" = '1' ] || return 0
	# shellcheck disable=SC2154
	filter['forbidden_url_local']="$(question_choice "Использовать заглушку на данном сервере?" "${filter['forbidden_url_local']}")"
	if [ "${filter['forbidden_url_local']}" = '1' ]; then
		#shellcheck disable=SC2154
		filter['forbidden_url']="$(ask_local_url)"
	else
		filter['forbidden_url']="$(ask_url_redirect)"
	fi
}

https() {
	show_msg "Настройка фильтрации HTTPS"
	# shellcheck disable=SC2154
	filter['sni']="$(question_choice "Фильтровать HTTPS по SNI" "1")"
	# shellcheck disable=SC2154
	filter['https_by_ip']="$(question_choice "Фильтровать HTTPS по IP?" "${filter['https_by_ip']}")"
	if [ "${filter['https_by_ip']}" = '1' ]; then
		# shellcheck disable=SC2154
		filter['https_by_nslookup']="$(question_choice "Использовать DNS-резолвер для расширения списка IP HTTPS-ресурсов?" "0")"
	fi
}

dns() {
	show_msg "Настройка фильтрации DNS"
	# shellcheck disable=SC2154
	filter['dns']="$(question_choice "Использовать фильтрацию DNS?" "1")"
	[ "${filter['dns']:-0}" = '1' ] || return 0
	# TODO: Возможно это ошибка насчёт двойного вопроса про forbidden_url_local
	if [ "${filter['forbidden_url_local']}" = '1' ]; then
		choose_dns_ip
		mkdir -p /etc/nginx/ssl/
		if [ ! -f /etc/nginx/ssl/cert.pem ] || [ ! -f /etc/nginx/ssl/cert.key ]; then
			openssl req -new -x509 -days 9999 -nodes -newkey rsa:2048 -out /etc/nginx/ssl/cert.pem -keyout /etc/nginx/ssl/cert.key
		fi
	else
		# shellcheck disable=SC2154
		filter['dns_ip']="$(question_inputbox "Настройка фильтрации" "Введите IP веб-сервера с заглушкой для редиректа" "$ip_regex" "${filter['dns_ip']}")"
	fi
}

proxy() {

	show_msg "Настройка интеграции с прокси-серверами."
	have_proxy="$(question_choice "У вас есть/будет прокси-сервер для фильтрации HTTPS-URL?" "0")"
	[ "$have_proxy" = '1' ] || return 0
	have_proxy_dns="$(question_choice 'Это специальный прокси-сервер от Carbon Soft с поддержкой отправки трафика с помощью DNS-спуфинга?' 0)"
	if [ "$have_proxy_dns" = '1' ] && [ "${filter['dns']}" = '1' ]; then
		# shellcheck disable=SC2154
		filter['dns_proxy_ip']="$(question_inputbox "Интеграция с прокси-серверами" "IP адрес прокси сервера от Carbon Soft" "$ip_regex" "${filter['dns_proxy_ip']}")"
		# shellcheck disable=SC2154
		filter['dns_proxy_hsts']="$(question_choice "Отправлять в прокси-сервер с помощью с помощью DNS ресурсы использующие HSTS?" '1')"
		# shellcheck disable=SC2154
		filter['dns_proxy_url_https']="$(question_choice "Отправлять в прокси-сервер с помощью DNS ВСЕ имена HTTPS-ресурсов, рекомендуемых к блокировке по URL?" '0')"
	fi
	have_proxy_bgp="$(question_choice 'Отправлять трафик в прокси-сервер с помощью BGP? (Потребуется интеграция)' 0)"
	if [ "$have_proxy_bgp" = '1' ]; then
		# shellcheck disable=SC2154
		filter['bgp_proxy_ip']="$(question_choice "IP адрес HTTPS-прокси сервера" "${filter['bgp_proxy_ip']}")"
		# shellcheck disable=SC2154
		filter['bgp_proxy_hsts']="$(question_choice "Отправлять в прокси-сервер по IP + резолву ресурсы использующие HSTS?" "${filter['bgp_proxy_hsts']}")"
		# shellcheck disable=SC2154
		filter['bgp_proxy_url_https']="$(question_choice "Отправлять в прокси-сервер по IP и резолву HTTPS-ресурсы, рекомендуемых к блокировке по URL?" "${filter['bgp_proxy_url_https']}")"
	fi
}

filter_info() {
	show_msg "Сейчас нужно будет настроить фильтрацию трафика.
Carbon Reductor предоставляет довольно много возможностей, для некоторых достаточно настроить зеркало трафика, для некоторых интегрировать другие узлы в сети провайдера.
Некоторые требуют использования нескольких серверов.

Итак, возможности (всё опционально):

Страница-заглушка может быть развёрнута:
- на отдельном веб-сервере с отдельным IP-адресом, доступном всем абонентам (рекомендуем)
- непосредственно на Carbon Reductor

Фильтрация HTTP с точностью до URL:
- с редиректом на страницу-заглушку по URL + разрывом TCP-соединения
- с разрывом TCP-соединения

Фильтрация HTTPS:
- по SNI заголовку (с точностью до домена) с разрывом TCP-соединения
- по IP адресу:
	- в случае, если иначе ресурс не заблокировать (<url>https://1.2.3.4/blabla</url>)
	- По списку IP адресов для всех HTTPS ресурсов, имеющихся в едином реестре.
	- По списку IP адресов, полученных с помощью системы периодического резолва доменов блокируемых HTTPS ресурсов.
- с точностью до URL при использовании прокси-сервера:
	- может использоваться как для всех HTTPS-ресурсов, добавленных в единый реестр как URL
	- может использоваться для фильтрации ссылок на HTTP(S)-ресурсы, использующие технологию HSTS, например youtube. Требует добавления корневого сертификата абонентами в список доверенных.
	- может использовать в качестве средства для отправки трафика: DNS-спуфинг (только прокси-сервер Carbon Soft)/BGP/OSPF (и/или).

Фильтрация DNS (по домену) с редиректом на страницу-заглушку по IP:
- с помощью модуля фильтрации, анализирующего зеркало трафика
- благодаря интеграции с DNS-серверами провайдера (сейчас поддерживаются Bind и Unbound)

Фильтрация прочих протоколов и по IP осуществляется двумя способами:
- По связкам IP + port, в ответ пользователю будет уходить TCP-reset или ICMP-ответ
- Анонсировать этот список IP по BGP/OSPF через Carbon Reductor."

	show_msg "Мы рекомендуем:

1. Настроить страницу-заглушку на отдельном IP, куда будут редиректить и HTTP и DNS фильтрации.
2. Включить DNS и SNI фильтрацию трафика.
3. Фильтровать HTTPS по IP в случае, если иначе это не заблокировать.
4. Провести интеграцию с маршрутизатором по BGP или иным способом для полной блокировки IP адресов.
5. Проинтегрировать Carbon Reductor с используемыми абонентами DNS серверами."
}

filter() {
	filter_info
	http
	https
	dns
	proxy
	save_and_exit
}

admins() {
	# shellcheck disable=SC2154
	admin['ip']="$(admin_ip)"
	# shellcheck disable=SC2154
	admin['web_ip']="$(admin_web_ip)"
	[ "${admin['web_port']}" = '80' ] && admin['web_port']='8080'
	admin['web_port']="$(admin_web_port)"
	save_and_exit
}

network() {
	local update_network="$(question_choice "Настроить сеть для приёма зеркала трафика с коммутатора или маршрутизатора?" "1")"
	[ "$update_network" = '0' ] || /usr/local/Reductor/bin/setup_network.sh
}

main() {
	registration
	rkn_settings
	filter
	admins
	network
	/etc/init.d/reductor restart
}

${1:-main}
