#!/bin/bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG
# shellcheck disable=SC1090
. $MENULIB
# shellcheck disable=SC1090
. $BINDIR/setup_network_lib.sh

remove_old_settings() {
	local bkdir defroute_dev file
	bkdir="$BACKUPDIR/network-scripts.$(date +%s)"
	mkdir -p "$bkdir"
	cp -varup $NETWORK_SCRIPTS/* "$bkdir"
	defroute_dev=$(get_defroute_dev)
	find $NETWORK_SCRIPTS/ ! -name "ifcfg-$defroute_dev" | egrep ifcfg-"(em|eth|br)[0-9\.]*" > /tmp/unused_configs
	while read file; do
		if [[ "$file" = *br* ]]; then
			ip link set dev ${file##*ifcfg-} down
			brctl delbr ${file##*ifcfg-}
		fi
		rm -f $file
	done < /tmp/unused_configs
}

make_bridge() {
	local bridge_tmplt=$MAINDIR/contrib/tmplts/ifcfg-bridge
	genconfig conf="$NETWORK_SCRIPTS/ifcfg-$1" tmplt=$bridge_tmplt ip="$(get_free_ip)" bridge="$1"
}

make_l3() {
	local device="$1"
	local ip="$2"
	conf=$NETWORK_SCRIPTS/ifcfg-$device
	tmplt=$MAINDIR/contrib/tmplts/ifcfg-eth-l3
	netmask="$(ipcalc --netmask $ip)"
	netmask_new="${netmask##*=}"
	ip="${ip%%/*}"
	genconfig conf=$conf tmplt=$tmplt eth=$device ip=$ip netmask=$netmask_new

}

make_l3_vlan() {
	local device="$1"
	local tag="$2"
	local ip="$3"
	local conf=$NETWORK_SCRIPTS/ifcfg-$device.$tag
	local tmplt=$MAINDIR/contrib/tmplts/ifcfg-eth-l3-vlan
	local netmask
	netmask="$(ipcalc --netmask $ip)"
	netmask="${netmask##*=}"
	ip="${ip%%/*}"
	genconfig conf=$conf tmplt=$tmplt eth=$device tag=$tag ip=$ip netmask=$netmask
}

make_l2_vlan() {
	local device="$1"
	local tag="$2"
	local bridge="$3"
	local conf=$NETWORK_SCRIPTS/ifcfg-$device.$tag
	local tmplt=$MAINDIR/contrib/tmplts/ifcfg-vlan
	local ip
	ip=$(get_free_ip)
	genconfig conf=$conf tmplt=$tmplt eth=$device tag=$tag ip=$ip bridge=$bridge
	make_bridge "$bridge"
}

make_l2_eth() {
	local device="$1"
	local bridge="$2"
	local conf=$NETWORK_SCRIPTS/ifcfg-$device
	local tmplt=$MAINDIR/contrib/tmplts/ifcfg-eth
	local ip
	ip=$(get_free_ip)
	genconfig conf=$conf tmplt=$tmplt eth=$device ip=$ip bridge=$bridge
	make_bridge $bridge
}

make_devices() {
	local device tag ip
	local br_num=-1
	while read device tag ip; do
		remove_dev_from_all_bridges "$device"
		if [ "$ip" = "-" ]; then
			((br_num++))
			if [ $tag = "-" ]; then
				make_l2_eth $device br$br_num
			else
				make_l2_vlan $device $tag br$br_num
			fi
		else
			if [ "$tag" = "-" ]; then
				make_l3 $device $ip
			else
				make_l3_vlan $device $tag $ip
			fi
		fi
	done < $USERDIR/mirror_info.conf
}

main() {
	remove_old_settings
	make_devices
	$BINDIR/autogit.sh /etc/sysconfig/network-scripts/ "after master setup" || true
}

${1:-main}
