#!/bin/bash

. /usr/local/Reductor/etc/const
. /etc/init.d/functions
# shellcheck disable=SC1090
. $MENULIB
# shellcheck disable=SC1090
. $CONFIG

set -eu

trap __exit EXIT

readonly MAINTAINANCE_REQUIRED="${MAINTAINANCE_REQUIRED:-0}"

check_network() {
	# shellcheck disable=SC2154
	[ "${misc['skipnetworkcheck']}" = '1' ] && return 0
	# shellcheck disable=SC2154
	[ "${filter['router']}" = '1' ] && return 0
	if ! brctl show | grep -q "br[0-9]"; then
		echo "Не настроена сеть."
		echo "Запустите команду 'menu' и выберите пункт:"
		echo "	Настройка сканирования трафика"
		echo "Либо, в случае если вы используете зеркало трафика с маршрутизатора, включите опцию"
		echo "	Прочие настройки ->"
		echo "	[ ] Не проверять настройки сети при старте"
		return 1
	fi
	return 0
}

check_and_install() {
	which "$@" >/dev/null || yum -y install "$@"
}
check_software() {
	check_and_install brctl
	check_and_install ebtables
	check_and_install dialog
	check_and_install wget
}

mount_work() {
	if [ ! -f $CHROOTDIR/dev/null ]; then
		mkdir -p $CHROOTDIR/dev
		mount -t devtmpfs none $CHROOTDIR/dev
	fi

	if [ ! -d $CHROOTDIR/proc/$$ ]; then
		mkdir -p $CHROOTDIR/proc
		mount -t proc none $CHROOTDIR/proc
	fi
}

check_reg_values() {
	[ "${autoupdate['operator']}" != 'Мегапровайдер' ]
	[ "${autoupdate['email']}" != 'master@example.com' ]
	if [ "${autoupdate['enabled']}" = '1' ]; then
		[ "${autoupdate['inn']}" != '1234567890' ]
		[ "${autoupdate['ogrn']}" != '1234567890123' ]
	fi
}

fix_link() {
	src=$1
	dst=$2
	if [ ! -L "$dst" ] || [ "$(readlink $dst)" != "$src" ]; then
		rm -f $dst
		ln -sf "$src" "$dst"
	fi
}

prepare() {
	for dir in $LOGDIR $TMPDIR /var/log/nginx; do
		if [ -f "$dir" ] || [ ! -d "$dir" ]; then
			rm -f "$dir"
			mkdir -vp "$dir"
		fi
	done

	# shellcheck disable=SC2154
	if [ "${reductorupdate['branch']:-master}" = 'devel' ]; then
		$BINDIR/developer_mode.sh || true
	fi
	if [[ "${autoupdate['url']}" = *docs.carbonsoft.ru* ]]; then
		autoupdate['url']='http://docs.carbonsoft.ru:1488/cache/'
		save_and_exit
	fi

	uname=$(uname -r)
	current_kernel=$MODULES/$uname
	if [ ! -d $current_kernel ]; then
		uname_regex="[0-9.el-]+x86_64"
		latest_kernel="$(find $MODULES -type f -name "xt_reductor.ko" -exec stat -c "%y %n" {} \; | sort -nr | head -1 | egrep -o "$uname_regex")"
		ln -s $latest_kernel $current_kernel
	fi
	[ ! -L /usr/bin/menu ] && ln -s ${BINDIR}/menu /usr/bin/menu
	fix_link $CHROOTDIR/lib64/xtables/libxt_reductor.so /lib64/xtables/libxt_reductor.so
	fix_link $CHROOTDIR/lib64/xtables/libxt_snimatch.so /lib64/xtables/libxt_snimatch.so
	fix_link $CHROOTDIR/lib64/xtables/libipt_FORBIDDEN.so /lib64/xtables/libipt_FORBIDDEN.so
	fix_link $CHROOTDIR/lib64/xtables/libip6t_FORBIDDEN.so /lib64/xtables/libip6t_FORBIDDEN.so
	fix_link $BINDIR/list_ctl /usr/local/bin/list_ctl
	fix_link $CHROOTDIR/lib64/xtables/libxt_dnsmatch.so /lib64/xtables/libxt_dnsmatch.so
	fix_link $CHROOTDIR/lib64/xtables/libipt_DNSREPLY.so /lib64/xtables/libipt_DNSREPLY.so
	cp -a /etc/resolv.conf $CHROOTDIR/etc/resolv.conf
	cp -a /etc/localtime $CHROOTDIR/etc/localtime
	cp -a $CONTRIB/usr/local/bin/reductor_revert.sh /usr/local/bin/reductor_revert.sh
}

checkers() {
	check_reg_values
	check_network
	check_software
}

replace_ifup() {
	mkdir -p $USERDIR/backups
	local postfix
	read postfix _ <<< "$(md5sum $NETWORK_SCRIPTS/ifup-eth)"
	cp -vu $NETWORK_SCRIPTS/ifup-eth $USERDIR/backups/ifup-eth.$postfix
	cp -a $CONTRIB/$NETWORK_SCRIPTS/ifup-eth $NETWORK_SCRIPTS/ifup-eth
}

reg_process() {
	$BINDIR/registration.sh migrate || return 1
	$BINDIR/registration.sh activation || return 2
	$BINDIR/load.sh
}

stability() {
	read panic < /proc/sys/kernel/panic
	if [ "$panic" == '0' ]; then
		echo 100 > /proc/sys/kernel/panic
	fi
	return 0
}

set_proc_values() {
	echo 1 > /proc/sys/net/ipv4/ip_forward
	# shellcheck disable=SC2044
	for entry in $(find /proc/sys/net/ipv4/conf -name "rp_filter"); do
		echo 2 > $entry
	done
	modprobe bridge
	echo 1 > /proc/sys/net/bridge/bridge-nf-call-iptables
	echo 1 > /proc/sys/net/ipv4/route/gc_min_interval
	echo 30 > /proc/sys/net/ipv4/route/gc_timeout
	echo 10000 > /proc/sys/net/ipv4/neigh/default/gc_thresh1
	echo 11000 > /proc/sys/net/ipv4/neigh/default/gc_thresh2
	echo 12000 > /proc/sys/net/ipv4/neigh/default/gc_thresh3
	stability
}

patch_crontab() {
	cp -up $CONTRIB/etc/cron.d/reductor /etc/cron.d/reductor
}

patch_sysctl_conf() {
	sed -i /etc/sysctl.conf -e 's/.*bridge-nf-call-iptables.*/net.bridge.bridge-nf-call-iptables = 1/g'
	sed -i /etc/sysctl.conf -e 's/net.ipv4.conf.default.rp_filter.*/net.ipv4.conf.default.rp_filter = 2/g'
	sed -i /etc/sysctl.conf -e 's/net.ipv4.ip_forward.*/net.ipv4.ip_forward = 1/g'
	sysctl -p > /dev/null
}

firewall() {
	$BINDIR/firewall.sh start
}

custom_things() {
	echo "- Дополнительные действия (при необходимости)"
	$BINDIR/billing_redirect.sh start
	$BINDIR/autorps_all.sh || true
	client_post_start_hook
}

client_post_start_hook() {
	return 0
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	if [ -x "$hook" ]; then
		# shellcheck disable=SC1090
		. "$hook"
	fi
}

web_server() {
	if $BINDIR/gen_nginx_conf.sh; then
		set +eu
		/etc/init.d/nginx restart &>$TMPDIR/nginx || cat $TMPDIR/nginx
		/etc/init.d/reductor_web restart &>$TMPDIR/uwsgi || cat $TMPDIR/uwsgi
		set -eu
	fi
	return 0
}

main() {
	log "# Запускается Carbon Reductor:"
	prepare
	mount_work
	checkers
	set_proc_values
	patch_sysctl_conf
	patch_crontab
	replace_ifup
	firewall
	web_server || true
	$BINDIR/update_signatures.sh migrate_only || true
	reg_process
	/etc/init.d/rknd start || true
	$BINDIR/maintain.sh unlock
	custom_things
}

use_hook
${1:-main}
