#!/bin/bash

. /usr/local/Reductor/etc/const

TMPFILE=$TMPDIR/hwinfo.tmp
LSPCI=$TMPDIR/lspci.tmp

add_lspci() {
	if [ ! -x /sbin/lspci ]; then
		yum -y install pciutils
	fi
}

grab_data() {
	for interface in /sys/class/net/*/brport/bridge; do
		if [ "$interface" = '/sys/class/net/*/brport/bridge' ]; then
			break
		fi
		bridge="$(readlink $interface | sed 's/.*\///g')"
		eth="$(cut -d / -f5 <<< $interface)"
		echo ${bridge#br} $eth
	done | sort -n | sed 's/^/br/' > $TMPFILE
	lspci > $LSPCI
}

if_ip() {
	ip -4 a show $1 | egrep -o $ip_regex/[0-9]+
}

bridges() {
	echo
	echo "# bridges"
	echo
	for bridge in $(cut -d ' ' -f1 $TMPFILE | uniq); do
		echo $bridge "$(if_ip $bridge)"
		for interface in $(grep -w $bridge $TMPFILE | cut -d ' ' -f2); do
			echo "- $interface $(if_ip $interface)"
		done
	done
}

devices() {
	echo
	echo "# devices"
	echo
	for eth in $(ip link | egrep -o "e(th|m)[0-9]+" | sort -u); do
		for id in $(ethtool -i $eth | grep bus-info | sed 's/.*0000://'); do
			rx_buf="$(ethtool -g $eth | tac | grep RX: | egrep -o [0-9]+ | tr '\n' '/'| sed 's|/$||g')"
			echo "$eth: $(grep -w $id $LSPCI) rx: $rx_buf"
		done
	done 
}

_uptime() {
	echo
	echo "# uptime"
	echo
	uptime
}

cpu() {
	echo
	echo "# cpu"
	echo
	lscpu
	lscpu -e
}

interrupts() {
	echo
	echo "# interrupts"
	echo
	grep eth /proc/interrupts
	echo
	echo "# softirq"
	echo
	grep NET_RX /proc/softirqs
}

config() {
	echo
	echo "# config"
	echo
	local useless='^(proc|monitoring|misc.diagnostic)'
	egrep "'[01]'" $CONFIG | tr -d "']" | tr '[' '.' | sed 's/=/ = /g' | sort -u | egrep -v "$useless"
}

branch_and_version() {
	echo
	echo "# branch"
	echo
	cat /usr/local/Reductor/etc/branch
	echo
	echo "# version_rpm"
	echo
	rpm -q reductor | tr -d '[a-z-]' | cut -d '.' -f1
	echo
	echo "# version_used"
	echo
	cat /usr/local/Reductor/etc/version
	echo
	echo "# version_web_rpm"
	echo
	rpm -q reductor_web | cut -d '-' -f2,3 | cut -d '.' -f 1,2,3 | tr -d '.-'
}

modules_state() {
	for module in xt_reductor xt_dnsmatch xt_snimatch; do
		file=/proc/net/${module/x/ip}/block_list
		[ -f $file ] || continue
		echo
		echo "# $module"
		echo
		cat $file
	done
}

lists_size() {
	echo
	echo "# Размеры списков"
	find $LISTDIR/{load,rkn,provider,carbonsoft,resolver} -type f -exec wc -l {} \; | sed "s|$LISTDIR/||g" | while read size list; do
		printf "%-40s %s\n" $list $size
	done
}

add_info() {
	echo
	echo "# Информация о провайдере"
	echo
	python -c "import json; print json.dumps(json.load(open('/var/reg/CRB-Reductor.reg')), indent=4, ensure_ascii=False).encode('utf-8')"
}

dfh() {
	echo
	echo "# Свободное место на файловых системах"
	echo
	df -h
}

main() {
	add_info
	add_lspci
	grab_data
	bridges
	devices
	_uptime
	cpu
	interrupts
	config
	branch_and_version
	modules_state
	lists_size
	dfh
}

"${@:-main}" | sed -e 's/^[^#$]/    &/g'
