#!/usr/bin/env bash

. /usr/local/Reductor/etc/const

set -eu

MIGRATE_VERSION="1"

__change_extension() {
	while read f; do mv -vf "$f" "${f/.$1/.$2}"; done
}

prepare() {
	mkdir -p $LISTDIR/{load,tmp,provider{,/unknown},carbonsoft,rkn,resolver}/
	touch /var/lib/reductor/migrate_version
	local migrate_version
	read migrate_version < /var/lib/reductor/migrate_version || migrate_version=0
	if [ "$migrate_version" -ge "$MIGRATE_VERSION" ]; then
		echo "Миграция $MIGRATE_VERSION не требуется"
		exit 0
	fi
}

move_to_new_dirs() {
	local ext
	local exts
	# ркн - возобновляемый источник. Стал сильно иначе парсится, необходимо переразобрать.
	# новый парсер делает это за 2 секунды. Не страшно.
	find $LISTDIR -maxdepth 1 -type f -name "rkn.*" -delete
	find $LISTDIR -maxdepth 1 -type f -name "system.*" -delete
	# так как всё равно всё переразбираем, а временные файлы не нужны - удаляем.
	declare -a exts=(
		load noload all entries wget nohttps fqdn domains without_domains with_sharp
		pages without_sharp pcap reduce fixed_sharp tmp sorted_1 fixed_sharp_old
		no_r processed nolongurl longurl old
	)
	for ext in "${exts[@]}"; do
		find $LISTDIR -maxdepth 1 -type f -name "*.$ext" -delete
	done
	# негласное соглашение - свои списки - our.*. Даже если больше не будут использоваться - муваем их в provider/
	find $LISTDIR -maxdepth 1 -type f -name "our.*" -exec mv "{}" $LISTDIR/provider/ \;
	# забираем свои списки в свою папку.
	find $LISTDIR -maxdepth 1 -type f -name "workaround.*" -exec mv "{}" $LISTDIR/carbonsoft/ \;
	find $LISTDIR -maxdepth 1 -type f -name "minjust.list" -exec mv "{}" $LISTDIR/carbonsoft/ \;
	find $LISTDIR -maxdepth 1 -type f -name "main.minjust" -delete
	# все остальные списки. Только те, что были в статистике за 24.10.2016.
	find $LISTDIR -maxdepth 1 -type f -name "*.list" -exec mv "{}" $LISTDIR/provider \;
	find $LISTDIR -maxdepth 1 -type f -name "*.iplist" -exec mv "{}" $LISTDIR/provider \;
	# мы не знаем что это такое. Пусть провайдер с техподдержкой разбираются, оставить это или удалить.
	find $LISTDIR -maxdepth 1 -type f -exec mv "{}" $LISTDIR/provider/unknown \;
}

remove_old_lists() {
	for i in dump_bad_lines main.minjust system.{https,ip6list,iplist,nohttps} http.ports https.resolv; do
		# keep symlink https.resolv for backward compat
		[ -L $LISTDIR/$i ] || rm -f $LISTDIR/$i
	done
}

change_extension() {
	local dir="$1"
	local src="$2"
	local dst="$3"
	find "$dir" -maxdepth 1 -type f -name "*.$src" | __change_extension "$src" "$dst"
}

# OLD                       | NEW                      |
# --------------------------+--------------------------+
# main.minjust             ->  rm -f
# minjust.list             -> carbonsoft/minjust.list
#
# gos_duma.list            -> provider/gosduma.url_http
# tambov.list              -> provider/tambov.url_http
# https.iplist             -> provider/https.ip_https
#
# our.list                 -> provider/our.url_http
# our.ip6list              -> provider/our.ip6_https
# our.whitelist            -> provider/our.url_http_whitelist
# our.white_https_iplist   -> provider/our.white_https_iplist
# our.iplist               -> provider/our.ip_https
# our.httpslist            -> provider/our.domain_mask
# our.fullwhitelist        -> mv provider/unknown/our.fullwhitelist + alert
# our.ipport               -> provider/our.ip_port
# our.whiteiplist          -> provider/our.ip_https_whitelist
# our.white_http_iplist    -> provider/our.white_http_iplist
# our.ip_block             -> provider/our.ip_block

new_legal_extensions() {
	for list in  urls domains ports ip; do
		$BINDIR/prepare_lists.sh --show-list $list | tr ' ' '\n'
		$BINDIR/prepare_lists.sh --show-list $list | tr ' ' '\n' | sed 's/.*/&_whitelist/'
	done
	echo white_http_iplist
	echo white_https_iplist
}

declare -A extensions_convert
extensions_convert['list']='url_http'
extensions_convert['iplist']='ip_https'
extensions_convert['ipport']='ip_port'
extensions_convert['ip6list']='ip6_https'
extensions_convert['httpslist']='domain_mask'
extensions_convert['whitelist']='url_http_whitelist'
extensions_convert['whiteiplist']='ip_https_whitelist'

change_extensions() {
	# тут всё очень сложно из-за ситуации с переименованиями белых списков. Но таки победим!
	local ext
	for ext in "${!extensions_convert[@]}"; do
		change_extension $LISTDIR/provider/ "$ext" "${extensions_convert[$ext]}"
	done
	new_legal_extensions > $TMPDIR/new_legal_extensions
	find $LISTDIR/provider/ -maxdepth 1 -type f  | egrep -v -f $TMPDIR/new_legal_extensions > $TMPDIR/bad_files || return 0
	log "Мы не знаем что делать с этими списками"
       	while read -r bad_file; do
		mv -v "$bad_file" $LISTDIR/provider/unknown/
	done < $TMPDIR/bad_files
}

main() {
	prepare
	log "Совершаем миграцию списков."
	remove_old_lists
	move_to_new_dirs
	change_extensions
	echo $MIGRATE_VERSION > /var/lib/reductor/migrate_version
	if [ "$MIGRATE_VERSION" -le '1' ] && [ -s $DUMPXML ]; then
		log "Так как изменился парсер реестра, необходимо переразобрать его."
		log "Саму выгрузку не перезапускаем так как слишком долго для хука."
		log "Точно так же не запускаем и резолвер, на 5 минут полагаемся на списки РКН/dns/sni"
		find $LISTDIR/rkn/ -type f -exec wc -l {} \; || true
		$BINDIR/dump_parser.sh
		$BINDIR/prepare_lists.sh
		$BINDIR/update_signatures.sh
	fi
}

main "$@"
