# coding:utf8
"""
Набор функций для регистрации Carbon Reductor
"""
__author__ = 'oleg'
import urllib
import urllib2
import json
from uuid import uuid1 as uuid
from time import sleep

REG_KEYWORDS = [
    'reg_num',
    'apps',
    'company_name',
    'email',
    'email_bookkeeper',
    'email_director',
    'phone',
    'phone_bookkeeper',
    'phone_director',
]
REG_KEYWORDS_OPTIONAL = [
    'email_bookkeeper',
    'email_director',
    'phone',
    'phone_bookkeeper',
    'phone_director',
]
REG_KEYWORDS_NON_ENV = ['reg_num', 'apps']
REG_KEYWORDS_REQUIRED = list(set(REG_KEYWORDS) - set(REG_KEYWORDS_OPTIONAL))
REG_FILE_PATH = '/var/reg/CRB-Reductor.reg'
KEY_FILE_PATH = '/var/reg/CRB-Reductor.key'
REG_SERVERS = [
    "http://helpdesk.carbonsoft.ru/registration/proxy.php"
]


def reg_num():
    """ Генерирует регномер формата xxx-xxx-xxx-xxx-xxx """
    uid = str(uuid().__int__())
    return "-".join([uid[i:i + 3] for i in xrange(0, 15, 3)])


def reg_data_new():
    """ Возвращает основную структуру данных о регистрации """
    return {
        "CRB-Reductor": {
            "apps": ["reductor"],
            "reg_num": str(reg_num()),
        }
    }


def reg_file_save(data=None):
    """ Сохраняет файл с данными регистрации """
    if not data:
        data = reg_data_new()
    with open(REG_FILE_PATH, 'w') as reg_file:
        reg_file.write(json.dumps(data, indent=4))


def reg_number_from_file():
    """ Нужно для того чтобы не перегенеривать регномер """
    with open(REG_FILE_PATH) as reg_file:
        data = json.load(reg_file)
        if data.get('CRB-Reductor'):
            if data['CRB-Reductor'].get('reg_num'):
                return data['CRB-Reductor']['reg_num']
    return None


def reg_file_read():
    """ возвращает данные регистрации в dict'е, читает из файла """
    with open(REG_FILE_PATH) as reg_file:
        return json.load(reg_file)


def get_hwid():
    """ Код установки """
    with open('/proc/net/ipt_reductor/block_list') as procfile:
        procfile_data = dict(line.split() for line in procfile.read().strip().split('\n'))
        return procfile_data.get('registration.hwid')


def reg_params(profile, reg_info, action):
    """ Все бесполезные преобразования и добавления данных в структурку """
    params = {}
    params['product'] = params['profile'] = profile
    params['hwid'] = get_hwid()
    params['cmd'] = action
    for key in REG_KEYWORDS:
        if not reg_info[profile].get(key):
            continue
        if key == 'apps':
            params[key] = reg_info[profile][key]
            continue
        params[key] = reg_info[profile][key].encode('utf-8')
    return params


def urlopen_with_retry(url, data, timeout, retry=5):
    """ Повторное обращение при плохом инете / недоступном сервере """
    for _try in range(retry, 0, -1):
        try:
            result = urllib2.urlopen(url, data, timeout)
            return result
        except urllib2.URLError:
            print "URLError, retry in ", str(timeout), "sec, ", \
                str(_try), " retry remains"
            sleep(timeout)
            continue


def __reg_file_send(params):
    """ Реальная отправка запроса и разбор ответа (надо бы разбить) """
    for reg_server in REG_SERVERS:
        params_encoded = urllib.urlencode(params)
        result = urlopen_with_retry(reg_server, params_encoded, 30)
        if not result:
            print 'No result from ' + reg_server
            continue
        result_data = json.loads(result.read())
        # will be very bad if where will be message in activation
        if result_data.get('CRB-Reductor') and result_data['CRB-Reductor'].get('msg'):
            print result_data['CRB-Reductor']['msg'].encode('utf-8')
        return json.dumps(result_data, indent=4)


def reg_file_send(action='activate'):
    """ Основная логика регистрации и активации """
    reg_info = reg_file_read()
    for profile in reg_info.keys():
        params = reg_params(profile, reg_info, action)
        return __reg_file_send(params)
